package GestaoBancos;

/**
 * Representa uma conta bancria de um cliente.
 * 
 * @author Joo Carlos Pascoal Faria
 * @version (30 de Janeiro de 2003)
 */
public class ContaBancaria
{
    // "Nested classes" para tratamento de excepes
    public static class ContaBancariaException extends Exception
    {
        public ContaBancariaException(String msg)
        {
            super(msg);
        }
    }

    public static class SaldoInsuficiente extends ContaBancariaException
    {
        public SaldoInsuficiente()
        {
            super("Saldo insuficiente");
        }
    }
    
    public static class SaldoNegativo extends ContaBancariaException
    {
        public SaldoNegativo()
        {
            super("Saldo negativo");
        }
    }    
    
    // instance variables
    private int numero;
    private Cliente cliente;
    private double saldo;
    
    // static variables
    static private int ultimoNumero = 0;

    /**
     * Constructor for objects of class ContaBancaria
     * 
     * @param cliente referencia para objecto Cliente (dono da conta)
     * @param saldo   saldo inicial
     */
    public ContaBancaria(Cliente cliente, double saldo) throws SaldoNegativo
    {
        // validaes
        if (saldo < 0)
            throw new SaldoNegativo();
            
        // atribui numero automaticamente
        ++ultimoNumero;
        numero = ultimoNumero;
        
        // guarda cliente
        this.cliente = cliente;
        
        // guarda saldo
        this.saldo = saldo;
    }

    /**
     * Constructor for objects of class ContaBancaria
     * 
     * @param cliente referncia para objecto Cliente (dono da conta)
     */
    public ContaBancaria(Cliente cliente) throws SaldoNegativo
    {
        // usa constructor anterior com saldo nulo
        this(cliente,0.0);
    }
    
     /**
     * Obtm cliente que  dono da conta
     * 
     * @return objecto Cliente que  dono da conta
     */
    public Cliente getCliente()
    {
        return cliente;
    }
    
     /**
     * Obtm nmero da conta
     * 
     * @return nmero da conta
     */
    public int getNumero()
    {
        return numero;
    }  
    
     /**
     * Obtm saldo da conta
     * 
     * @return saldo da conta
     */
    public double getSaldo()
    {
        return saldo;
    }    
    
     /**
     * Deposita valor na conta
     * 
     * @param valor valor a depositar
     */
    public void depositar(double valor)
    {
        saldo += valor;
    }     
    
     /**
     * Retira valor da conta
     * 
     * @param valor valor a levantar
     */
    public void levantar(double valor) throws SaldoInsuficiente
    {
        if (saldo < valor)
            throw new SaldoInsuficiente();
        saldo -= valor;
    }     
    
    public static void transferir(double valor, ContaBancaria conta1, ContaBancaria conta2) throws SaldoInsuficiente
    {
        conta1.levantar(valor);
        conta2.depositar(valor);
    }
}
