
import java.util.Vector;

/**
 * Representa um cluster.
 * 
 * @author Joo Carlos Pascoal Faria 
 * @version 6 de Fevereiro de 2003
 */
public class Cluster
{
    /**
     * Conjunto de elementos do cluster representado com a classe Vector do java
     */
    private Vector elements = new Vector();
    
    /**
     * Distncia mxima especificada aquando da criao do cluster.
     * A distncia entre qualquer ponto e o ponto mais prximo
     * no pode exceder esta distncia mxima.
     */
    private double maxDistance;
    
    /**
     * Constri um cluster caracterizado por uma distancia mxima,
     * mas inicialmente vazio.
     * 
     * @param maxDistance distancia mxima entre qualquer ponto e 
     *                    o ponto mais prxima
     */
    public Cluster(double maxDistance)
    {
        this.maxDistance = maxDistance;
    }

    /**
     * Verifica se um determinado elemento pode ser adicionado ao cluster,
     * isto , se o ponto mais prximo est a uma distncia que no excede
     * a distncia mxima especificada no constructor.
     *  sempre possivel adicionar um ponto a um cluster vazio.
     * 
     * @param elem elemento a adicionar (de uma subclasse concreta de ClusterElement)     
     * @return booleano com o resultado
     */
    public boolean mayAddElement(ClusterElement elem)
    {
       if (elements.size() == 0)
            return true;
            
       for (int i = 0; i < elements.size(); i++)
            if ( elem.distanceTo((ClusterElement)elements.elementAt(i)) <= maxDistance)
                return true;
                
       return false;
    } 
    
    /**
     * Adiciona um elemento ao cluster.
     * 
     * @param  elem  elemento a adicionar (de uma subclasse concreta de ClusterElement)     
     */
    public void addElement(ClusterElement elem)
    {
        elements.addElement(elem);
    }

    /**
     * Adiciona todos os elementos de outro cluster a este cluster.
      * 
     * @param  cluster2 o outro cluster     
     */    
    public void addCluster(Cluster cluster2)
    {
        for (int i = 0; i < cluster2.elements.size(); i++)
            this.addElement((ClusterElement)cluster2.elements.elementAt(i));
    }
    

     /**
     * Representa um cluster numa string no formato {elem1, elem2, ...., elemk}
     * @return a string pretendida
     */
    public String toString()
    {
        String s = "{";
        for (int i=0; i < elements.size(); i++)
        {
            if (i > 0)
                s = s + ", ";
            s = s + elements.elementAt(i);
        }    
        s = s + "}";
        return s;
     }
}
